@@ -24,6 +24,9 @@ pub struct framebuffer {
   ptr: (bool, *mut u8),
 }
 
+unsafe impl Send for framebuffer {}
+unsafe impl Sync for framebuffer {}
+
 impl Drop for framebuffer {
   fn drop(&mut self) { if self.ptr.0 { free(self.ptr_mut(), self.len()); } }
 }
@@ -57,6 +60,14 @@ pub mod colors {
   #[repr(C)] #[derive(Copy, Clone)] pub struct rgb { pub r: u8, pub g: u8, pub b: u8 }
   #[repr(C)] #[derive(Copy, Clone)] pub struct rgba { pub r: u8, pub g: u8, pub b: u8, pub a: u8 }
 
+  impl From<rgba> for rgb {
+    fn from(c: rgba) -> rgb { return rgb { r: c.r, g: c.g, b: c.b }; }
+  }
+
+  impl From<rgb> for rgba {
+    fn from(c: rgb) -> rgba { return rgba { r: c.r, g: c.g, b: c.b, a: 255 }; }
+  }
+
   impl From<rgb> for u32 {
     fn from(c: rgb) -> u32 { return c.r as u32 | ((c.g as u32) << 8) | ((c.b as u32) << 16) | (255 << 24); }
   }
@@ -65,6 +76,14 @@ pub mod colors {
     fn from(c: rgba) -> u32 { return c.r as u32 | ((c.g as u32) << 8) | ((c.b as u32) << 16) | ((c.a as u32) << 24); }
   }
 
+  impl From<u32> for rgb {
+    fn from(c: u32) -> rgb { return rgb { r: (c & 0xFF) as u8, g: ((c >> 8) & 0xFF) as u8, b: ((c >> 16) & 0xFF) as u8 }; }
+  }
+
+  impl From<u32> for rgba {
+    fn from(c: u32) -> rgba { return rgba { r: (c & 0xFF) as u8, g: ((c >> 8) & 0xFF) as u8, b: ((c >> 16) & 0xFF) as u8, a: (c >> 24) as u8 }; }
+  }
+
   pub fn blend(bg: u32, fg: u32) -> u32 {
     let fa = fg >> 24;
     let alpha = 1 + fa;
